/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2010 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.designer.models.transformers.bpmn2designer;

import java.util.HashMap;
import java.util.Map;

import javax.xml.bind.JAXBElement;
import javax.xml.namespace.QName;

import org.ow2.orchestra.designer.bpmn.model.AbstractElement;
import org.ow2.orchestra.designer.bpmn.model.ProcessModel;
import org.ow2.orchestra.jaxb.bpmn.Definitions;
import org.ow2.orchestra.jaxb.bpmn.TFlowElement;
import org.ow2.orchestra.jaxb.bpmn.TProcess;
import org.ow2.orchestra.jaxb.bpmn.bpmndi.BPMNEdge;
import org.ow2.orchestra.jaxb.bpmn.bpmndi.BPMNPlane;
import org.ow2.orchestra.jaxb.bpmn.bpmndi.BPMNShape;
import org.ow2.orchestra.jaxb.bpmn.di.DiagramElement;

/**
 * Class holding the context of a transformation from a BPMN model into a
 * Designer model.
 *
 * @author Loic Albertin
 */
public final class Bpmn2DesignerContext {

  /**
   * The original {@link ProcessModel}.
   */
  private ProcessModel processModel;

  /**
   * The corresponding {@link Definitions}.
   */
  private Definitions definitions;

  /**
   * A {@link Map} between {@link TFlowElement} and the corresponding
   * {@link AbstractElement}. This map is populated when transforming a
   * {@link TFlowElement} into an {@link AbstractElement}
   */
  private final Map<TFlowElement, AbstractElement> modelsMap = new HashMap<TFlowElement, AbstractElement>();

  /**
   * A {@link Map} between {@link TFlowElement}'s {@link QName}s and the
   * corresponding {@link DiagramElement}s.
   */
  private final Map<QName, DiagramElement> diagramElementsMap = new HashMap<QName, DiagramElement>();

  /**
   * A {@link Map} between {@link TFlowElement}'s {@link QName}s and the
   * corresponding {@link TFlowElement}s.
   */
  private final Map<QName, TFlowElement> flowElementsMap = new HashMap<QName, TFlowElement>();

  /**
   * @return the processModel
   */
  public ProcessModel getProcessModel() {
    return this.processModel;
  }

  /**
   * @param processModel the processModel to set
   */
  public void setProcessModel(final ProcessModel processModel) {
    this.processModel = processModel;
  }

  /**
   * @return the modelsMap
   */
  public Map<TFlowElement, AbstractElement> getModelsMap() {
    return this.modelsMap;
  }

  /**
   * @return the diagramElementsMap
   */
  public Map<QName, DiagramElement> getDiagramElementsMap() {
    return this.diagramElementsMap;
  }

  /**
   * @return the flowElementsMap
   */
  public Map<QName, TFlowElement> getFlowElementsMap() {
    return this.flowElementsMap;
  }

  /**
   * @return the definitions
   */
  public Definitions getDefinitions() {
    return this.definitions;
  }

  /**
   * @param definitions the definitions to set
   */
  public void setDefinitions(final Definitions definitions) {
    this.definitions = definitions;
  }

  /**
   * Populate the diagramElementsMap with the diagram elements contained into
   * the plane
   *
   * @param plane the plane containing {@link DiagramElement}s
   */
  public void initiateDiagramElementMap(final BPMNPlane plane) {
    for (final JAXBElement<? extends DiagramElement> jaxbElement : plane.getDiagramElements()) {
      if (jaxbElement.getValue() instanceof BPMNEdge) {
        final BPMNEdge edge = (BPMNEdge) jaxbElement.getValue();
        this.diagramElementsMap.put(edge.getBpmnElement(), edge);
      } else if (jaxbElement.getValue() instanceof BPMNShape) {
        final BPMNShape shape = (BPMNShape) jaxbElement.getValue();
        this.diagramElementsMap.put(shape.getBpmnElement(), shape);
      } else if (jaxbElement.getValue() instanceof BPMNPlane) {
        final BPMNPlane includedPlane = (BPMNPlane) jaxbElement.getValue();
        this.initiateDiagramElementMap(includedPlane);
      }
    }
  }

  /**
   * Populate the flowElementsMap with the elements contained in
   * the process
   *
   * @param process the process containing {@link TFlowElement}s
   */
  public void initiateFlowElementMap(final TProcess process) {
    for (final JAXBElement<? extends TFlowElement> jaxbElement : process.getFlowElements()) {
      final TFlowElement flowElement = jaxbElement.getValue();
      this.flowElementsMap.put(new QName(this.getDefinitions().getTargetNamespace(), flowElement.getId()), flowElement);
    }
  }
}
