/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2010 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.designer.models.transformers.bpmn2designer;

import java.util.List;

import javax.xml.namespace.QName;

import org.ow2.orchestra.designer.bpmn.model.AbstractElement;
import org.ow2.orchestra.designer.bpmn.model.AbstractElementWithPosition;
import org.ow2.orchestra.designer.bpmn.model.SequenceFlowModel;
import org.ow2.orchestra.designer.models.exceptions.TransformationException;
import org.ow2.orchestra.jaxb.bpmn.TEvent;
import org.ow2.orchestra.jaxb.bpmn.TFlowElement;
import org.ow2.orchestra.jaxb.bpmn.TFlowNode;
import org.ow2.orchestra.jaxb.bpmn.TGateway;
import org.ow2.orchestra.jaxb.bpmn.TSequenceFlow;
import org.ow2.orchestra.jaxb.bpmn.TTask;

/**
 * Utility class used to retrieve the correct transformer for a given object
 *
 * @author Loic Albertin
 */
public final class ElementTransformer {

  /**
   * Utility classes should not have public or default constructors.
   */
  private ElementTransformer() {
  }


  /**
   * Transform an {@link TFlowElement} into a {@link AbstractElement}. this
   * method mainly retrieve the correct transformer and call its bpmnToModel
   * method.
   *
   * @param element the element to transform
   * @param context the transformation context
   * @return Returns true if the transformation succeeded
   * @throws TransformationException Throws {@link TransformationException} if
   *                                 any of sub-transformers throws a {@link TransformationException}
   */
  public static void bpmnToModel(final TFlowElement element, final Bpmn2DesignerContext context) throws TransformationException {
    if (!context.getModelsMap().containsKey(element)) {
      if (element instanceof TGateway) {
        final TGateway tGateway = (TGateway) element;
        GatewayTransformer.bpmnToModel(tGateway, context);
      } else if (element instanceof TSequenceFlow) {
        final TSequenceFlow tSequenceFlow = (TSequenceFlow) element;
        SequenceFlowTransformer.bpmnToModel(tSequenceFlow, context);
      } else if (element instanceof TEvent) {
        final TEvent tEvent = (TEvent) element;
        EventTransformer.bpmnToModel(tEvent, context);
      } else if (element instanceof TTask) {
        final TTask tTask = (TTask) element;
        TaskTransformer.bpmnToModel(tTask, context);
      }
      if (element instanceof TFlowNode) {
        ElementTransformer.bpmnToModelIncomingOutgoingSequenceFlows((TFlowNode) element, context);
      }
    }
    // Designer model represent a sub-part of BPMN model so
    // some BPMN elements doesn't have a transformation into Designer model.
  }

  public static void bpmnToModelIncomingOutgoingSequenceFlows(final TFlowNode bpmnElement, final Bpmn2DesignerContext context) throws TransformationException {
    final AbstractElement contextElement = context.getModelsMap().get(bpmnElement);
    if (contextElement instanceof AbstractElementWithPosition) {
      final AbstractElementWithPosition modelElement = (AbstractElementWithPosition) contextElement;
      final List<QName> incomingSequenceFlowRefs = bpmnElement.getIncomings();
      if (incomingSequenceFlowRefs != null) {
        for (final QName incomingSequenceFlowRef : incomingSequenceFlowRefs) {
          // Get incomingSequenceFlow
          final TFlowElement incomingElement = context.getFlowElementsMap().get(incomingSequenceFlowRef);
          // Generate model for incoming sequenceFlow
          ElementTransformer.bpmnToModel(incomingElement, context);
          // get from bpmn model and add to designer model
          final AbstractElement abstractElement = context.getModelsMap().get(incomingElement);
          if (abstractElement instanceof SequenceFlowModel) {
            modelElement.getIncomingSequenceFlows().add((SequenceFlowModel) abstractElement);
          } else {
            throw new TransformationException("BPMN conversion error: incoming element " + incomingSequenceFlowRef + " of " + bpmnElement.getId()
                + " does not reference a valid SequenceFlow.");
          }
        }
      }
      final List<QName> outgoingSequenceFlowRefs = bpmnElement.getOutgoings();
      if (outgoingSequenceFlowRefs != null) {
        for (final QName outgoingSequenceFlowRef : outgoingSequenceFlowRefs) {
          // Get SequenceFlow
          final TFlowElement outgoingElement = context.getFlowElementsMap().get(outgoingSequenceFlowRef);
          // Generate model for sequenceFlow
          ElementTransformer.bpmnToModel(outgoingElement, context);
          // get from bpmn model and add to designer model
          final AbstractElement abstractElement = context.getModelsMap().get(outgoingElement);
          if (abstractElement instanceof SequenceFlowModel) {
            modelElement.getOutgoingSequenceFlows().add((SequenceFlowModel) abstractElement);
          } else {
            throw new TransformationException("BPMN conversion error: incoming element " + outgoingSequenceFlowRef + " of " + bpmnElement.getId()
                + " does not reference a valid SequenceFlow.");
          }
        }
      }
    }
  }

}
